@@ -19,7 +19,7 @@ import re
 from .utils import F
 from . import utils
 
-__version__ = '1.0'
+__version__ = '1.1.0'
 
 logger = logging.getLogger(__name__)
 
@@ -146,6 +146,7 @@ class CrossEnvBuilder(venv.EnvBuilder):
 
         self.find_host_python(host_python)
         self.find_compiler_info()
+        self.get_uname_info()
 
         super().__init__(
                 system_site_packages=False,
@@ -315,16 +316,17 @@ class CrossEnvBuilder(venv.EnvBuilder):
         if not os.path.exists(self.host_makefile):
             raise FileNotFoundError("Cannot find Makefile")
 
-        self.host_platform = sys.platform # Default: not actually cross compiling
+        # Default: not actually cross compiling
+        self.host_platform = sysconfig.get_platform() 
         with open(self.host_makefile, 'r') as fp:
             lines = list(fp.readlines())
 
         for line in lines:
             line = line.strip()
             if line.startswith('_PYTHON_HOST_PLATFORM='):
-                host_platform = line.split('=',1)[-1]
+                host_platform = line.split('=',1)[-1].strip()
                 if host_platform:
-                    self.host_platform = line.split('=',1)[-1]
+                    self.host_platform = host_platform
                 break
 
         self.macosx_deployment_target = ''
@@ -409,7 +411,6 @@ class CrossEnvBuilder(venv.EnvBuilder):
 
         env_dir = os.path.abspath(env_dir)
         context = self.ensure_directories(env_dir)
-        self.create_configuration(context)
         self.make_build_python(context)
         self.make_cross_python(context)
         self.post_setup(context)
@@ -443,27 +444,29 @@ class CrossEnvBuilder(venv.EnvBuilder):
         utils.mkdir_if_needed(context.lib_path)
         return context
 
-    def create_configuration(self, context):
+    def get_uname_info(self):
         """
-        Create configuration files. We don't have a pyvenv.cfg file in the
-        base directory, but we do have a uname crossenv.cfg file.
+        What should uname() return?
         """
 
-        # Do our best to guess defaults
-        config = ConfigParser()
         # host_platform is _probably_ something like linux-x86_64, but it can
         # vary.
         host_info = self.host_platform.split('-')
         if not host_info:
-            sysname = sys.platform
+            self.host_sysname = sys.platform
+            self.host_machine = platform.machine()
         elif len(host_info) == 1:
-            sysname = host_info[0]
-            machine = platform.machine()
+            self.host_sysname = host_info[0]
+            self.host_machine = platform.machine()
+        elif host_info[-1] == "powerpc64le":
+            # On uname.machine=ppc64le, _PYTHON_HOST_PLATFORM is linux-powerpc64le
+            self.host_sysname = host_info[0]
+            self.host_machine = "ppc64le"
         else:
-            sysname = host_info[0]
-            machine = host_info[-1]
+            self.host_sysname = host_info[0]
+            self.host_machine = host_info[-1]
 
-        release = ''
+        self.host_release = ''
         if self.macosx_deployment_target:
             try:
                 major, minor = self.macosx_deployment_target.split(".")
@@ -472,25 +475,13 @@ class CrossEnvBuilder(venv.EnvBuilder):
                 raise ValueError("Unexpected value %s for MACOSX_DEPLOYMENT_TARGET" %
                         self.macosx_deployment_target)
             if major == 10:
-                release = "%s.0.0" % (minor + 4)
+                self.host_release = "%s.0.0" % (minor + 4)
             elif major == 11:
-                release = "%s.0.0" % (minor + 20)
+                self.host_release = "%s.0.0" % (minor + 20)
             else:
                 raise ValueError("Unexpected major version %s for MACOSX_DEPLOYMENT_TARGET" %
                         major)
 
-        config['uname'] = {
-            'sysname' : sysname.title(),
-            'nodename' : 'build',
-            'release' : release,
-            'version' : '',
-            'machine' : machine,
-        }
-
-        context.crossenv_cfg = os.path.join(context.env_dir, 'crossenv.cfg')
-        with utils.overwrite_file(context.crossenv_cfg) as fp:
-            config.write(fp)
-
     def make_build_python(self, context):
         """
         Assemble the build-python virtual environment
@@ -501,7 +492,8 @@ class CrossEnvBuilder(venv.EnvBuilder):
         env = venv.EnvBuilder(
                 system_site_packages=self.build_system_site_packages,
                 clear=self.clear_build,
-                with_pip=self.with_build_pip)
+                with_pip=self.with_build_pip,
+                symlinks=True)
         env.create(context.build_env_dir)
         context.build_bin_path = os.path.join(context.build_env_dir, 'bin')
         context.build_env_exe = os.path.join(
@@ -580,11 +572,16 @@ class CrossEnvBuilder(venv.EnvBuilder):
                 with_pip=False)
         env.create(context.cross_env_dir)
         context.cross_bin_path = os.path.join(context.cross_env_dir, 'bin')
+        context.cross_lib_path = os.path.join(context.cross_env_dir, 'lib')
         context.cross_env_exe = os.path.join(
                 context.cross_bin_path, context.python_exe)
         context.cross_cfg_path = os.path.join(context.cross_env_dir, 'pyvenv.cfg')
         context.cross_activate = os.path.join(context.cross_bin_path, 'activate')
 
+        pyver = 'python' + sysconfig.get_config_var('py_version_short')
+        context.cross_site_lib_path = os.path.join(context.cross_lib_path,
+                pyver, 'site-packages')
+
         # Remove binaries. We'll run from elsewhere
         for exe in os.listdir(context.cross_bin_path):
             if not exe.startswith('activate'):
@@ -604,9 +601,6 @@ class CrossEnvBuilder(venv.EnvBuilder):
 
         # Note that env_exe hasn't actually been created yet.
 
-        sysconfig_name = os.path.basename(self.host_sysconfigdata_file)
-        sysconfig_name, _ = os.path.splitext(sysconfig_name)
-
         # If this venv is generated from a cross-python still in its
         # build directory, rather than installed, then our modifications
         # prevent build-python from finding its pure-Python libs, which
@@ -645,20 +639,48 @@ class CrossEnvBuilder(venv.EnvBuilder):
             else:
                 extra_envs.append(('CPATH', ':=', inc))
 
-        utils.install_script('pywrapper.py.tmpl', context.cross_env_exe, locals())
+        # Put a few things in locals to make templating marginally less gross
+        macosx_deployment_target = self.macosx_deployment_target
+        host_sysconfigdata = self.host_sysconfigdata
+        host_build_time_vars = self.host_sysconfigdata.build_time_vars
+        sysconfig_name = self.host_sysconfigdata_name
 
+ 
+        # Install patches to environment
+        self.copy_and_patch_sysconfigdata(context)
+
+        tmpl = utils.TemplateContext()
+        tmpl.update(locals())
+
+        utils.install_script('pywrapper.py.tmpl', context.cross_env_exe, tmpl)
+
+        # Everything in lib_path follows the same pattern
+        site_scripts = [
+            'site.py',
+            'sys-patch.py',
+            'os-patch.py',
+            'importlib-machinery-patch.py',
+            'platform-patch.py',
+            'sysconfig-patch.py',
+            'distutils-sysconfig-patch.py',
+            'pkg_resources-patch.py',
+        ]
+
+        for script in site_scripts:
+            src = script + '.tmpl'
+            dst = os.path.join(context.lib_path, script)
+            utils.install_script(src, dst, tmpl)
+
+        utils.install_script('_manylinux.py.tmpl',
+                os.path.join(context.cross_site_lib_path, '_manylinux.py'),
+                tmpl)
+
+        # Symlink alternate names to our wrapper
         for exe in ('python', 'python3'):
             exe = os.path.join(context.cross_bin_path, exe)
             if not os.path.exists(exe):
                 utils.symlink(context.python_exe, exe)
 
-        macosx_deployment_target = self.macosx_deployment_target
-        # Install patches to environment
-        utils.install_script('site.py.tmpl',
-                os.path.join(context.lib_path, 'site.py'),
-                locals())
-        self.copy_and_patch_sysconfigdata(context)
-
         # cross-python is ready. We will use build-pip to install cross-pip
         # because 'python -m ensurepip' is likely to get confused and think
         # that there's nothing to do.
@@ -683,8 +705,8 @@ class CrossEnvBuilder(venv.EnvBuilder):
         sysconfig_name = os.path.basename(self.host_sysconfigdata_file)
         # we always write a .py, but we might be reading from a .pyc
         # (i.e., from buildroot).
-        sysconfig_name = os.path.splitext(sysconfig_name)[0] + '.py'
-        cross_sysconfig = os.path.join(context.lib_path, sysconfig_name)
+        sysconfig_name = self.host_sysconfigdata_name + '.py'
+        context.cross_sysconfig = os.path.join(context.lib_path, sysconfig_name)
 
         # Patch all instances of CC, etc. We'll do a global search and
         # replace
@@ -719,7 +741,7 @@ class CrossEnvBuilder(venv.EnvBuilder):
 
         cross_sysconfig_data['build_time_vars'] = build_time_vars
 
-        with open(cross_sysconfig, 'w') as fp:
+        with open(context.cross_sysconfig, 'w') as fp:
             fp.write("# generated from %s\n" % self.host_sysconfigdata_file)
             for key, value in cross_sysconfig_data.items():
                 fp.write("%s = " % key)
@@ -730,9 +752,11 @@ class CrossEnvBuilder(venv.EnvBuilder):
         Extra processing. Put scripts/binaries in the right place.
         """
 
+        tmpl = utils.TemplateContext()
+        tmpl.update(locals())
         utils.install_script('cross-expose.py.tmpl',
                 os.path.join(context.bin_path, 'cross-expose'),
-                locals())
+                tmpl)
 
         # Don't trust these to be symlinks. A symlink to Python will mess up
         # the virtualenv.
